﻿/**
******************************************************************************
* @file    ControlCAN.cs
* @author  lys
* @version V1.1.0
* @date    2023
* @brief  
*
* @note
*
* @endverbatim
*
******************************************************************************
* 版权:长沙六叶树教育科技有限公司
* 官网:www.liuyeshu.cn
* 更多资料教程下载见官网
******************************************************************************
修改日期    版本号   修改者   功能描述
2023.05.12 V1.1.0  LYS
----------------------------------------------------------------------------
******************************************************************************
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace ControlCAN
{
    //设备信息的数据类型
    [StructLayout(LayoutKind.Sequential)]
    public  struct  LCAN_BOARD_INFO
    {
        public UInt16	hw_Version;//硬件版本
        public UInt16	fw_Version;//固件版本
        public UInt16	dr_Version;//驱动程序版本
        public UInt16	in_Version;//接口库版本
        public UInt16	irq_Num;   //板卡所使用的中断号
        public byte	    can_Num;   //CAN通道数
        //序列号
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 20)]
        public byte[] 	serialNumber;
        //硬件型号
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 40)]
        public byte[] 	str_hw_Type;
        //保留
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public byte[] reserved;
    }

    //CAN信息帧的数据类型。
    [StructLayout(LayoutKind.Sequential)]
    public struct LCAN_CAN_OBJ
    {
        public uint id;
        public uint timeStamp;
        public byte timeFlag;
        public byte sendType;
        public byte remoteFlag;//是否是远程帧
        public byte externFlag;//是否是扩展帧
        public byte dataLen;

        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public byte[] reserved;

        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public byte[] data;
    }

    //FD信息帧的数据类型。
    [StructLayout(LayoutKind.Sequential)]
    public struct LCAN_CANFD_OBJ
    {
        public uint id;
        public uint timeStamp;
        public byte flag;//CANFD_BRS/ESI
        public byte sendType;
        public byte remoteFlag;//是否是远程帧
        public byte externFlag;//是否是扩展帧
        public byte dataLen;

        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public byte[] reserved;

        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 64)]
        public byte[] data;
    }

    //定义CAN的配置信息 用于CAN设备
    [StructLayout(LayoutKind.Sequential)]
    public struct LCAN_INIT_CONFIG
    {
        public UInt32 accCode;
        public UInt32 accMask;
        public UInt32 reserved;
        public byte filter;
        public byte timing0;
        public byte timing1;
        public byte mode;
    }

    //定义CANFD的配置信息 用于FD设备
    [StructLayout(LayoutKind.Sequential)]
    public struct LCAN_INITFD_CONFIG
    {
        public UInt32 abitBaudHz;
        public UInt32 dbitBaudHz;
        public UInt32 abit_timing;
        public UInt32 dbit_timing;
        public byte mode;
        public byte fdEn;
        public byte isoEn;
        public byte rev2;
    }

    //定义过滤器配置信息
    public struct LCAN_FLITER_CONFIG 
    {
        public byte cmd;
        public byte fliterMode;
        public byte idType;
        public byte rev1;
        public UInt32 fliterStardId;
        public UInt32 fliterEndId;
    }

    //
    public struct LCAN_Device
    {
        public const UInt32 LCAN_USBCAN1 = 3;
        public const UInt32 LCAN_USBCAN2 = 4;
        public const UInt32 LCAN_USBCANFDMini = 43;
        public const UInt32 LCAN_USBCANFD1 = 42;
        public const UInt32 LCAN_USBCANFD2 = 41;

        [DllImport("Controlcan.dll")]
        public static extern UInt32 LCAN_OpenDevice(UInt32 devType, UInt32 devIndex);
        [DllImport("Controlcan.dll")]
        public static extern UInt32 LCAN_CloseDevice(UInt32 devType, UInt32 devIndex);
        [DllImport("Controlcan.dll")]
        public static extern UInt32 LCAN_GetReceiveNum(UInt32 devType, UInt32 devIndex, UInt32 canIndex, byte type);
        [DllImport("Controlcan.dll", CharSet = CharSet.Ansi)]
        public static extern UInt32 LCAN_Receive(UInt32 devType, UInt32 devIndex, UInt32 canIndex, IntPtr pReceive, UInt32 len, Int32 waitTime);
        [DllImport("Controlcan.dll", CharSet = CharSet.Ansi)]
        public static extern UInt32 LCAN_ReceiveFD(UInt32 devType, UInt32 devIndex, UInt32 canIndex, IntPtr pReceive, UInt32 len, Int32 waitTime);
        [DllImport("Controlcan.dll")]
        public static extern UInt32 LCAN_Transmit(UInt32 devType, UInt32 devIndex, UInt32 canIndex, LCAN_CAN_OBJ[] pSend, UInt32 len);
        [DllImport("Controlcan.dll")]
        public static extern UInt32 LCAN_TransmitFD(UInt32 devType, UInt32 devIndex, UInt32 canIndex, LCAN_CANFD_OBJ[] pSend, UInt32 len);
        [DllImport("Controlcan.dll")]
        public static extern UInt32 LCAN_InitCAN(UInt32 devType, UInt32 devIndex, UInt32 canIndex, ref LCAN_INIT_CONFIG pInitConfig);
        [DllImport("Controlcan.dll")]
        public static extern UInt32 LCAN_InitCANFD(UInt32 devType, UInt32 devIndex, UInt32 canIndex, ref LCAN_INITFD_CONFIG pInitConfig);
        [DllImport("Controlcan.dll")]
        public static extern UInt32 LCAN_SetFliter(UInt32 devType, UInt32 devIndex, UInt32 canIndex, ref LCAN_FLITER_CONFIG pFliterConfig);
    };

    // 函数调用返回状态值
    public struct LCAN_STATUS
    {
        public const UInt32 STATUS_OK = 1;
        public const UInt32 STATUS_ERR = 0;
    };

    //FD FLAG
    public struct LCAN_FD_FLAG
    {
        public const byte LCAN_CANFD_BRS = 0x08;
        public const byte LCAN_CANFD_ESI = 0x01;
    };

    //FLITER CMD
    public struct LCAN_FLITER_CMD
    {
        public const byte LCAN_CANFD_ESI = 0x01;
        public const byte LCAN_FLITER_CMD_CLR = 0x00;
        public const byte LCAN_FLITER_CMD_ADD = 0x01;
        public const byte LCAN_FLITER_CMD_ENABLE = 0x02;
    };

    //DATA DLC
    public struct LCAN_DATA_DLC
    {
        public const UInt32 CAN_DATA_DLC_MAX = 8;
        public const UInt32 CANFD_DATA_DLC_MAX = 64;
    };

}/*namespace ControlCAN*/